/***************************************************************************
 *   Copyright (C) 2006 by Sergejs Melderis   *
 *   smelderis@yahoo.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "textencoder.h"
#include <qapplication.h>
TextEncoder::TextEncoder()
{
    QString windows_1251_chars[] = {
        "C0", "C1", "C2", "C3", "C4", "C5", "C6", "C7", "C8", "C9",
        "CA", "CB", "CC", "CD", "CE", "CF", "D0", "D1", "D2", "D3",
        "D4", "D5", "D6", "D7", "D8", "D9", "DA", "DB", "DC", "DD",
        "DE", "DF", "E0", "E1", "E2", "E3", "E4", "E5", "E6", "E7",
        "E8", "E9", "EA", "EB", "EC", "ED", "EE", "EF", "F0", "F1",
        "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "FA", "FB",
        "FC", "FD", "FE", "FF", "20"};

    QChar unicode_chars[] = {
        0x0410, 0x0411, 0x0412, 0x0413, 0x0414, 0x0415, 0x0416, 0x0417, 0x0418, 0x0419,
        0x041A, 0x041B, 0x041C, 0x041D, 0x041E, 0x041F, 0x0420, 0x0421, 0x0422, 0x0423,
        0x0424, 0x0425, 0x0426, 0x0427, 0x0428, 0x0429, 0x042A, 0x042B, 0x042C, 0x042D,
        0x042E, 0x042F, 0x0430, 0x0431, 0x0432, 0x0433, 0x0434, 0x0435, 0x0436, 0x0437,
        0x0438, 0x0439, 0x043A, 0x043B, 0x043C, 0x043D, 0x043E, 0x043F, 0x0440, 0x0441,
        0x0442, 0x0443, 0x0444, 0x0445, 0x0446, 0x0447, 0x0448, 0x0449, 0x044A, 0x044B,
        0x044C, 0x044D, 0x044E, 0x044F, 0x0020};

    for (int i = 0; i < 65; i++) {
        m_uniTo1251Map.insert(unicode_chars[i], windows_1251_chars[i]);
    }

    QString chars[142] = {
        /*
        QString(QChar(0x0442)).append("j"), QString(QChar(0x0442)).append(QChar(0x044C)), //тj -> ть
        QString(QChar(0x043D)).append("j"), QString(QChar(0x043D)).append(QChar(0x044C)), //нj -> нь
        QString(QChar(0x043C)).append("j"), QString(QChar(0x043C)).append(QChar(0x044C)), //мj -> мь
        QString(QChar(0x0441)).append("j"), QString(QChar(0x0441)).append(QChar(0x044C)), //сj -> сь
        QString(QChar(0x0437)).append("j"), QString(QChar(0x0437)).append(QChar(0x044C)), //зj -> зь
        QString(QChar(0x0432)).append("j"), QString(QChar(0x0432)).append(QChar(0x044C)), //вj -> вь
        QString(QChar(0x043B)).append("j"), QString(QChar(0x043B)).append(QChar(0x044C)), //лj -> ль
        QString(QChar(0x0434)).append("j"), QString(QChar(0x0434)).append(QChar(0x044C)), //дj -> дь
        QString(QChar(0x0440)).append("j"), QString(QChar(0x0440)).append(QChar(0x044C)), //рj -> рj
        QString(QChar(0x044C)).append("u"), QChar(0x044E),
        */
        "-e", QChar(0x044D),
        "-E", QChar(0x042D),
        "-i", QChar(0x044B),
        "-I", QChar(0x042B),
        QString(QChar(0x0441)).append(QChar(0x0441)).append("h"), QChar(0x0449),
        QString(QChar(0x0421)).append(QChar(0x0421)).append("h"), QChar(0x0429),
        QString(QChar(0x0421)).append(QChar(0x0421)).append("H"), QChar(0x0429),
        QString(QChar(0x0441)).append("h"), QChar(0x0448),
        QString(QChar(0x0421)).append("h"), QChar(0x0428),
        QString(QChar(0x0446)).append("h"), QChar(0x0447),
        QString(QChar(0x0426)).append("h"), QChar(0x0427),
        QString(QChar(0x0439)).append("u"), QChar(0x044E),
        QString(QChar(0x0419)).append("u"), QChar(0x042E),
        QString(QChar(0x0439)).append("o"), QChar(0x0451),
        QString(QChar(0x0439)).append("a"), QChar(0x044f),
        QString(QChar(0x0419)).append("a"), QChar(0x042F),
        QString(QChar(0x0437)).append("h"), QChar(0x0436),
        QString(QChar(0x0417)).append("h"), QChar(0x0416),
        QString(QChar(0x0439)).append("/").append("o"), QString(QChar(0x0439)).append(QChar(0x043E)),
        "a" , QChar(0x0430),
        "A" , QChar(0x0410),
        "b" , QChar(0x0431),
        "B" , QChar(0x0411),
        "c" , QChar(0x0446),
        "C" , QChar(0x0426),
        "d" , QChar(0x0434),
        "D" , QChar(0x0414),
        "e" , QChar(0x0435),
        "E" , QChar(0x0415),
        "f" , QChar(0x0444),
        "F" , QChar(0x0424),
        "g" , QChar(0x0433),
        "G" , QChar(0x0413),
        "h" , QChar(0x0445),
        "H" , QChar(0x0425),
        "l" , QChar(0x043B),
        "L" , QChar(0x041B),
        "i" , QChar(0x0438),
        "I" , QChar(0x0418),
        "k" , QChar(0x043A),
        "K" , QChar(0x041A),
        "j" , QChar(0x0439),
        "J" , QChar(0x0419),
        "m" , QChar(0x043C),
        "M" , QChar(0x041C),
        "n" , QChar(0x043D),
        "N" , QChar(0x041D),
        "o" , QChar(0x043E),
        "O" , QChar(0x041E),
        "p" , QChar(0x043F),
        "P" , QChar(0x041F),
        "r" , QChar(0x0440),
        "R" , QChar(0x0420),
        "s" , QChar(0x0441),
        "S" , QChar(0x0421),
        "t" , QChar(0x0442),
        "T" , QChar(0x0422),
        "u" , QChar(0x0443),
        "U" , QChar(0x0423),
        "z" , QChar(0x0437),
        "Z" , QChar(0x0417),
        "v" , QChar(0x0432),
        "V" , QChar(0x0412),
        "w" , QChar(0x0449),
        "W" , QChar(0x0429),
        "x" , QChar(0x0445),
        "X" , QChar(0x0425),
        "'" , QChar(0x044C),
        "\"", QChar(0x044A),
        "#" , QChar(0x044A),
        " " , QChar(0x0020)
    };
    for (int i = 0; i < 142; i += 2) {
        m_enToRusList.append(qMakePair(chars[i], chars[i+1]));
    }
}


TextEncoder::~TextEncoder()
{
}

QString TextEncoder::uniTo1251(const QString &uniText)
{
    QString _1251Text;
    int length = uniText.length();
    for (int i = 0; i < length; i++) {
        if (m_uniTo1251Map.contains(uniText.at(i))) {
            _1251Text.append("%");
            _1251Text.append(m_uniTo1251Map[uniText.at(i)]);
        }
    }
    return _1251Text;
}

QString TextEncoder::englishToRussian(const QString &enText)
{
    QString text = enText;
    QValueList<QPair<QString, QString> >::iterator iter;
    for (iter = m_enToRusList.begin(); iter != m_enToRusList.end(); iter++) {
        text.replace((*iter).first, (*iter).second);
    }
    return text;
}

QString TextEncoder::accentsToLingvoEncoding(const QString &acText)
{
    // I don't know what kind of encoding is that but that's what
    // Lingvo uses.
    QString _acText = acText;
    int l = acText.length();
    for (int i = 0; i < l; i++) {
        if (_acText.at(i).unicode() >= 192 && _acText.at(i).unicode() <= 252) {
            _acText.replace(i, 1, "%26%23" +
                    QString::number(_acText.at(i).unicode()) + "%3B");
        }
    }
    return _acText;
}





